----------------------------------------------------------------------------------------------
--	This class adds anchor point functionality to Sprites, Shapes and all of your own classes!
--	Simply call AnchorPoint( ClassName ) and all the necessary methods will be added.
--
--	The class was created for Gideros Illustrator: go.to/gideros-illustrator
--	by jack0088 [@gmx.de]
--
--	This class wouldn't exist without @ar2rsawseen and his phenomenal mind! [appcodingeasy.com]
----------------------------------------------------------------------------------------------
local function AnchorPoint( Class )
	
	local constructor = Class.new --cache
 
	function Class.new()
		
		local instance = constructor() --call default class behavior
		
		
		--chache original functionality
		local __setX = instance.setX
		local __setY = instance.setY
		local __posX = instance.getX
		local __posY = instance.getY
		local __getX = function(self) return Helper:round( __posX(self) ,2 ) end
		local __getY = function(self) return Helper:round( __posY(self) ,2 ) end
		local __setRotation = instance.setRotation
		local __setScaleX   = instance.setScaleX
		local __setScaleY   = instance.setScaleY
		
		
		instance.anchorPoint = { --default (TopLeft)
			cx = 0,
			cy = 0,
			offX = 0,
			offY = 0
		}
		
		
		function instance:getAnchorPointOffsetX()
			return self.anchorPoint.offX
		end
		
		function instance:getAnchorPointOffsetY()
			return self.anchorPoint.offY
		end
		
		function instance:setAnchorPointOffset( ox, oy )
			self.anchorPoint.offX = ox
			self.anchorPoint.offY = oy
		end
		
		
		function instance:setAnchorPoint( cx, cy )
			
			--[[
					1	cache Rotation
					2	save AnchorPoint
					3	move Object back to its REAL position (position withOUT the AnchorPoint)
					4	set AnchorPoint Offset
					5	refresh the Object's position with the new AnchorPoint Offset
					6	restore Rotation
					
					Setting Rotation is always the last operation.
			--]]
			
			--[[ 1 ]] local rotation = self:getRotation() ; if rotation ~= 0 then self:setRotation(0) end --chache Rotation
			
			
			--[[ 2 ]] self.anchorPoint.cx, self.anchorPoint.cy = cx, cy
			
			
			--[[ 3 ]] self:setPosition(self:getX() + self:getAnchorPointOffsetX(), self:getY() + self:getAnchorPointOffsetY())
			
			--[[ 4 ]] self:setAnchorPointOffset(Helper:round( self:getWidth() * self.anchorPoint.cx ,2 ), Helper:round( self:getHeight() * self.anchorPoint.cy ,2 ))
			
			--[[ 5 ]] self:setPosition( __getX(self), __getY(self) )
			
			
			--[[ 6 ]] if rotation ~= 0 then self:setRotation(rotation) end --restore Rotation, if any
		end
		
		function instance:getAnchorPoint()
			return self.anchorPoint.cx, self.anchorPoint.cy
		end
		
		
		function instance:setX( x )
			__setX( self, x - self:getAnchorPointOffsetX() )
		end
		
		function instance:setY( y )
			__setY( self, y - self:getAnchorPointOffsetY() )
		end
		
		
		function instance:getX()
			return __getX(self) + self:getAnchorPointOffsetX()
		end
		
		function instance:getY()
			return __getY(self) + self:getAnchorPointOffsetY()
		end
		
		
		function instance:setPosition( x, y )
			self:setX(x)
			self:setY(y)
		end
		
		function instance:getPosition()
			return self:getX(), self:getY()
		end
		
		
		function instance:setRotation( n )
			local offX 	 = self:getAnchorPointOffsetX()
			local offY 	 = self:getAnchorPointOffsetY()
			local cosine = math.cos(math.rad(n))
			local sine 	 = math.sin(math.rad(n))
			local dx 	 = Helper:round( offX - (offX * cosine - offY * sine) ,2 ) --x,y difference to apply
			local dy 	 = Helper:round( offY - (offY * cosine + offX * sine) ,2 )
			
			__setRotation(self, n)
			
			self:setAnchorPointOffset(offX - dx, offY - dy)
			self:setPosition(self:getX() + dx, self:getY() + dy)
		end
		
		
		function instance:setScaleX( n )
			__setScaleX(self, n)
			self:setAnchorPoint(self:getAnchorPoint())
		end
		
		function instance:setScaleY( n )
			__setScaleY(self, n)
			self:setAnchorPoint(self:getAnchorPoint())
		end
		
		function instance:setScale( nx, ny )
			self:setScaleX(nx)
			self:setScaleY(ny)
		end
		
		
		return instance
		
	end
	
end


--apply to classes
AnchorPoint( Sprite )
AnchorPoint( Shape  )