Helper = Core.class()


--[[
	 Expected table:
	 ´´´´´´´´´´´´´´´
	 Value = {			-- table
		[i] = {			-- index
			x = %d,		-- (x|y) values
			y = %d
		},
		...
	 }
	
	 Output (resulting) table:
	 ´´´´´´´´´´´´´´´´´´´´´´´´´
	 Value = {
		[i] = {
			%d,		-- all values are listed sequent. first comes x then y.
			%d,
			...
		}
	 }
	 makes serial list of points
]]
function Helper:serializePoints( vertices )
	local container = {}
		
	for i=1, #vertices do
		table.insert(container, vertices[i].x)
		table.insert(container, vertices[i].y)
	end
	
	return container
end


--[[ The opposite of serializePoints() ]]
function Helper:de_serializePoints( vertices )
	local count, container = 0, {}
	
	if #vertices ~= 1 then -- no (x|y) needed, when only one value
		
		for i=1, #vertices do
			count = count+1
			if  count == 2 then
				container[#container].y = vertices[i]
				count = 0
			else
				table.insert( container, { x = vertices[i] } )
			end
		end
		
	else
		container = vertices
	end
	
	return container
end


--[[ Returns given numbers with custom decimal count ]]
function Helper:round( n, decimals )
	return math.floor( (n * 10^decimals) + 0.5) / (10^decimals)
end


function Helper:random()
	local oldrandom = math.random
	local randomtable
	
	local function randomness()
		if  randomtable == nil then
			randomtable =  {}
			for i=1, 97 do
				randomtable[i] = oldrandom()
			end
		end
		local x = oldrandom()
		local i = 1 + math.floor(97*x)
		x, randomtable[i] = randomtable[i], x
		return x
	end
	return string.sub(randomness(), 3)
end