Primitive = Core.class( Shape )


----------------------------------------------------------------------------------------------------
--[[ PRIVATE! ]]
----------------------------------------------------------------------------------------------------
--[[ clear Shape coordinates ]]
function Primitive:_clearShapeCoordinates()
	self.Gestalt.coordinates = {}
end

function Primitive:_deleteGestalt()
	self.Gestalt = nil
end



----------------------------------------------------------------------------------------------------
--[[ PUBLIC ]]
----------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------
--[[ >>> Constructor: ]]
----------------------------------------------------------------------------------------------------
function Primitive:init()
	-- calling Primitive.new() is equivalent to Shape.new()
	
	
	--[[ Shape and all its properties! ]]
	self.Gestalt = {
		coordinates = {},   -- (x|y) coordinates that make the shape
		isPolygon 	= false,-- Flag. Closed paths on TRUE or "isPolygon". Opened paths (Polyline) on FALSE.
		style = { 			-- default styling values
			line = {
				width = 1,
				color = "0x000000",
				alpha = 1
			},
			fill = {
				["type"] = Shape.NONE,
				color	 = "0xffffff",
				alpha	 = 1
			}
		}
	}
end


----------------------------------------------------------------------------------------------------
--[[ >>> Shape appearance options: ]]
----------------------------------------------------------------------------------------------------
function Primitive:drawShape( tag )
	-- Update isPolygon tag!
	if tag == "isPolygon" then tag = true end --pass ("isPolygon"|true|false) or leave blank!
	if tag == nil then tag = false end
	   self.Gestalt.isPolygon  = tag
	
	-- Clear last drawing:
	self:clear()
	
	-- Re-Draw:
	self:beginPath( Shape.NON_ZERO )
		
		-- Style:
		self:setLineStyle( self.Gestalt.style.line.width, self.Gestalt.style.line.color, self.Gestalt.style.line.alpha )
		self:setFillStyle( self.Gestalt.style.fill.type,  self.Gestalt.style.fill.color, self.Gestalt.style.fill.alpha )
		
		-- Make:
		self:moveTo( self.Gestalt.coordinates[1].x, self.Gestalt.coordinates[1].y )
		for i=1, #self.Gestalt.coordinates do
			self:lineTo( self.Gestalt.coordinates[i].x, self.Gestalt.coordinates[i].y )
		end
	
	-- This endings depend on isPolygon tag of the self.Gestalt!
	if self.Gestalt.isPolygon == true then
		self:closePath()
	end
	self:endPath()
	
	-- Clear Gestalt attributes:
	--clearShapeCoordinates() --clears only the Shape coodrinates
	self:_deleteGestalt()   --deletes everything, also the Shape coordinates
end

function Primitive:setStyle( t ) -- [lineWidth] [,lineColor] [,lineAlpha] [,fillType] [,fillColor] [,fillAlpha]
	if t.fillColor and t.fillType == nil then t.fillType = Shape.SOLID end -- fillColor was given, but no fillType -> allow filling
	
	self.Gestalt.style = {
		line = {
			width = t.lineWidth or self.Gestalt.style.line.width,
			color = t.lineColor or self.Gestalt.style.line.color,
			alpha = t.lineAlpha or self.Gestalt.style.line.alpha
		},
		fill = {
			["type"] = t.fillType  or self.Gestalt.style.fill.type,
			color	 = t.fillColor or self.Gestalt.style.fill.color,
			alpha	 = t.fillAlpha or self.Gestalt.style.fill.alpha
		}
	}
end


function Primitive:getSize()
	return self:getWidth(), self:getHeight()
end
function Primitive:setWidth ( height )
	self:setScaleX(1) -- Um originale Breite und Höhe, ohne Skalierung, zu erhalten!
	self:setScaleX( height / self:getWidth() )
end
function Primitive:setHeight( width )
	self:setScaleY(1)
	self:setScaleY( width / self:getHeight() )
end
function Primitive:setSize( width, height )
	self:setWidth( width )
	self:setHeight( height )
end

----------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------
function Primitive:addPoints( vertices ) -- t{x, y, ...} OR t{[i] = {x, y}, ...}
	-- Errors. Check, Rebuild vertices to match the needs!
	if type(vertices) ~= "table" then
		error("table expected, got "..type(vertices))
	else
		if type(vertices[1]) == "number" then -- Try first argument. If number, than its a list-styled-table.
			-- Rebuild table to our needs, Apply changes:
			vertices = Helper:de_serializePoints( vertices )
		end
		-- Save given points:
		for l=1, #vertices do
			table.insert( self.Gestalt.coordinates, { x = vertices[l].x, y = vertices[l].y } )
		end
		
		-- Return for potential use:
		--print_r(vertices)
		return vertices
	end
end

----------------------------------------------------------------------------------------------------
--[[ >>> Shape variety: ]]
----------------------------------------------------------------------------------------------------
function Primitive:newPolyline( ... ) -- t{x, y, ...}
	self:addPoints( ... ) -- save points	
	self:drawShape() -- draw
end

function Primitive:newLine( x1, y1, x2, y2 ) -- x1, y1, x2, y2
	self:newPolyline( {x1, y1, x2, y2} ) -- made from Polyline
end

function Primitive:newPolygon( ... ) -- t{x, y, ...}
	-------------------------------------------------------------------------------------
	-- For differences between polygons and paths, read inside self:newPath()!
	-- Differences between newPolyline and newPolygon is the drawing-tag. I had to write this functions separately because of the cleanup at the end!
	-------------------------------------------------------------------------------------
	
	self:addPoints( ... ) -- save points
	self:drawShape( "isPolygon" ) -- draw
end

function Primitive:newRect( x, y, width, height ) -- x, y, width, height
	self:newPolygon( {0,0, width,0, width,height, 0,height} )
	self:setPosition( x, y )
end

function Primitive:newPath( dstring, pieceLenght ) --[[ svg d="string" expected -> the only logical way of manually defining paths
																   tolerance			   -> distance between points, where granularity should be applied
																   granularity			   -> how pricise the path shoul be ]]
	-------------------------------------------------------------------------------------
	--[[ Info ! ]]
	--[[ Paths are based on polylines.
		 Paths, can be opened or closed.
			From now on a CLOSED PATH will be called a POLYGON!
			From now on a OPENED PATH will be called a POLYLINE!
		Paths may have Bezier-Curves - polygons doesnt.
			From now on a Bezier Curve will be called a PATH!
	]] ----------------------------------------------------------------------------------
	
	local Bezier = Bezier.new( dstring, pieceLenght )
	
	self:addPoints( Bezier.vertices )
	
	self:drawShape( Bezier.isPolygon )
end

function Primitive:newUniformPolygon( x, y, radius, points ) -- x, y ,radius|radii [,points]
	----------------------------------------------------------------------------------------------------
	-- This function can create everything in between a triangle to a perfect circle.
	-- Streched shapes (defined by two circle-radii) are also common.
	----------------------------------------------------------------------------------------------------
	
	-- If no points given, define them.
	-- Check also whether points is points or was intended to be left blank. If so, then this is the style.
	if  points == nil then
		points =  360 / 10 -- 360° / x° = points (e.g. 360/10 = 36 points)
	elseif points <= 2 then
		error("minimum 3 points expected, got only "..points)
	end
	
	
	-- Maths:
	local Sinus, Cosinus, Pi = math.sin, math.cos, math.pi -- chache!
	local angle = 2 * Pi / points
	
		-- Save:
		for i=1, points do
			if type(radius) == "table" then
				-- mix points (x from rx and y from ry) to get intersection point at Sp(rx.x, ry.y)
				-- (for ellipses and this kind of shapes where radius[x|y] is NOT equal)
				self:addPoints{ radius.rx*Cosinus(i*angle), radius.ry*Sinus(i*angle) }
			else
				-- save r.x and r.y
				-- (for all uniform shapes where radius[x|y] is equal)
				self:addPoints{ radius*Cosinus(i*angle) + radius, radius*Sinus(i*angle) + radius } --"+radius" brings the Origin (center of the Shape) to the TopLeft, to match all the other Shapes!
			end
		end
	
	-- Make:
	self:drawShape(true) -- ("isPolygon|true")
	
	self:setPosition(x, y)
end

function Primitive:newCircle( ... ) -- x, y, radius [,points]
	self:newUniformPolygon( ... )
end

function Primitive:newEllipse( x, y, rx, ry, points ) -- x, y, rx, ry [,points]
	self:newUniformPolygon( x, y, {rx = rx, ry = ry}, points )
end
